# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/headers/http/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "Header",
    "Method",
    "RequestConfig",
    "ResponseConfig",
    "Status",
    "Version",
)

from dataclasses import dataclass

import betterproto2

from ......message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    request: "RequestConfig | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for authenticating requests. If not set, client side will not send
    authentication header, and server side will bypass authentication.
    """

    response: "ResponseConfig | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for authenticating responses. If not set, client side will bypass
    authentication, and server side will not send authentication header.
    """


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "Config", Config
)


@dataclass(eq=False, repr=False)
class Header(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    "Accept", "Cookie", etc
    """

    value: "list[str]" = betterproto2.field(2, betterproto2.TYPE_STRING, repeated=True)
    """
    Each entry must be valid in one piece. Random entry will be chosen if
    multiple entries present.
    """


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "Header", Header
)


@dataclass(eq=False, repr=False)
class Method(betterproto2.Message):
    """
    HTTP method. Default value "GET".
    """

    value: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "Method", Method
)


@dataclass(eq=False, repr=False)
class RequestConfig(betterproto2.Message):
    version: "Version | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Full HTTP version like "1.1".
    """

    method: "Method | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    GET, POST, CONNECT etc
    """

    uri: "list[str]" = betterproto2.field(3, betterproto2.TYPE_STRING, repeated=True)
    """
    URI like "/login.php"
    """

    header: "list[Header]" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "RequestConfig", RequestConfig
)


@dataclass(eq=False, repr=False)
class ResponseConfig(betterproto2.Message):
    version: "Version | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    status: "Status | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    header: "list[Header]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "ResponseConfig", ResponseConfig
)


@dataclass(eq=False, repr=False)
class Status(betterproto2.Message):
    code: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    Status code. Default "200".
    """

    reason: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    Statue reason. Default "OK".
    """


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "Status", Status
)


@dataclass(eq=False, repr=False)
class Version(betterproto2.Message):
    """
    HTTP version. Default value "1.1".
    """

    value: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.headers.http", "Version", Version
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/observatory/burst/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "HealthPingConfig",
)

from dataclasses import dataclass

import betterproto2

from ......message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    subject_selector: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )
    """
    @Document The selectors for outbound under observation
    """

    ping_config: "HealthPingConfig | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.core.app.observatory.burst", "Config", Config
)


@dataclass(eq=False, repr=False)
class HealthPingConfig(betterproto2.Message):
    destination: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    destination url, need 204 for success return
    default https://connectivitycheck.gstatic.com/generate_204
    """

    connectivity: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)
    """
    connectivity check url
    """

    interval: "int" = betterproto2.field(3, betterproto2.TYPE_INT64)
    """
    health check interval, int64 values of time.Duration
    """

    sampling_count: "int" = betterproto2.field(4, betterproto2.TYPE_INT32)
    """
    sampling count is the amount of recent ping results which are kept for calculation
    """

    timeout: "int" = betterproto2.field(5, betterproto2.TYPE_INT64)
    """
    ping timeout, int64 values of time.Duration
    """


default_message_pool.register_message(
    "xray.core.app.observatory.burst", "HealthPingConfig", HealthPingConfig
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: common/log/log.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Severity",)


import betterproto2

betterproto2.check_compiler_version("0.5.1")


class Severity(betterproto2.Enum):
    Unknown = 0

    Error = 1

    Warning = 2

    Info = 3

    Debug = 4



from grpclib.exceptions import GRPCError

from xray_cfo.xray.app.router.command import (
    AddRuleRequest,
    GetBalancerInfoRequest,
    OverrideBalancerTargetRequest,
    RemoveRuleRequest,
    RoutingContext,
    TestRouteRequest,
)
from xray_cfo.xray.common.serial import TypedMessage

from .._base import BaseService


class RoutingService(BaseService):

    async def subscribe_routing_stats(self, *selectors: str):
        try:
            # return (
            #     await self.routing_stub.subscribe_routing_stats(
            #         SubscribeRoutingStatsRequest(
            #             field_selectors=list(selectors)
            #         )
            #     )
            # ).stat.value
            pass
        except GRPCError:
            return

    async def test_route(
        self, context: RoutingContext, *selectors: str, publish_result: bool = True
    ):
        try:
            return await self.routing_stub.test_route(
                TestRouteRequest(
                    routing_context=context,
                    field_selectors=list(selectors),
                    publish_result=publish_result,
                )
            )
        except GRPCError:
            return None

    async def get_balancer_info(self, tag: str):
        try:
            return (
                await self.routing_stub.get_balancer_info(
                    GetBalancerInfoRequest(
                        tag=tag,
                    )
                )
            ).balancer
        except GRPCError:
            return None

    async def override_balancer_target(self, balancer_tag: str, target: str):
        try:
            await self.routing_stub.override_balancer_target(
                OverrideBalancerTargetRequest(
                    balancer_tag=balancer_tag,
                    target=target,
                )
            )
            return True
        except GRPCError:
            return None

    async def add_rule(self, config: TypedMessage, should_append: bool = True):
        try:
            await self.routing_stub.add_rule(
                AddRuleRequest(
                    config=config,
                    should_append=should_append,
                )
            )
            return True
        except GRPCError as e:
            print(e.status)
            print(e.message)
            print(e.details)
            return None

    async def remove_rule(self, rule_tag: str):
        try:
            await self.routing_stub.remove_rule(
                RemoveRuleRequest(
                    rule_tag=rule_tag,
                )
            )
            return True
        except GRPCError:
            return None



from decimal import Decimal
from enum import StrEnum

from pydantic import Field

from xray_cfo.models.xray.enums import Network

from ._base import XBase


class DomainStrategy(StrEnum):
    AsIs = "AsIs"
    IPIfNonMatch = "IPIfNonMatch"
    IPOnDemand = "IPOnDemand"


class DomainMatcher(StrEnum):
    hybrid = "hybrid"
    linear = "linear"


class Protocol(StrEnum):
    http = "http"
    tls = "tls"
    quic = "quic"
    bittorrent = "bittorrent"


class Rule(XBase):
    domain_matcher: DomainMatcher = Field(
        DomainMatcher.hybrid, serialization_alias="domainMatcher"
    )
    type: str = "field"
    domain: list[str] | None = None
    ip: list[str] | None = None
    port: str | int = None
    sourcePort: str | int = None
    network: Network | None = None
    source: list[str] | None = None
    user: list[str] | None = None
    inboundTag: list[str] | None = None
    protocol: list[Protocol] | None = None
    attrs: dict[str, str] | None = None
    outboundTag: str | None = None
    balancerTag: str | None = None
    ruleTag: str | None = None


class StrategyType(StrEnum):
    random = "random"
    roundRobin = "roundRobin"
    leastPing = "leastPing"
    leastLoad = "leastLoad"


class Cost(XBase):
    regexp: bool
    match: str
    value: Decimal


class StrategySettings(XBase):
    expected: int
    maxRTT: str
    tolerance: Decimal
    baselines: list[str]
    costs: list[Cost]


class Strategy(XBase):
    type: StrategyType
    settings: StrategySettings


class Balancer(XBase):
    tag: str
    selector: list[str] | None = None
    fallbackTag: str
    strategy: list[Strategy]


class XRouting(XBase):
    domain_strategy: DomainStrategy = Field(
        DomainStrategy.AsIs, serialization_alias="domainStrategy"
    )
    domain_matcher: DomainMatcher = Field(
        DomainMatcher.hybrid, serialization_alias="domainMatcher"
    )
    rules: list[Rule] | None = None
    balancers: list[Balancer] | None = None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AddressPortStrategy",
    "DomainStrategy",
    "SocketConfigTProxyMode",
    "CustomSockopt",
    "ProxyConfig",
    "SocketConfig",
    "StreamConfig",
    "TransportConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class AddressPortStrategy(betterproto2.Enum):
    None_ = 0

    SrvPortOnly = 1

    SrvAddressOnly = 2

    SrvPortAndAddress = 3

    TxtPortOnly = 4

    TxtAddressOnly = 5

    TxtPortAndAddress = 6


class DomainStrategy(betterproto2.Enum):
    AS_IS = 0

    USE_IP = 1

    USE_IP4 = 2

    USE_IP6 = 3

    USE_IP46 = 4

    USE_IP64 = 5

    FORCE_IP = 6

    FORCE_IP4 = 7

    FORCE_IP6 = 8

    FORCE_IP46 = 9

    FORCE_IP64 = 10


class SocketConfigTProxyMode(betterproto2.Enum):
    Off = 0
    """
    TProxy is off.
    """

    TProxy = 1
    """
    TProxy mode.
    """

    Redirect = 2
    """
    Redirect mode.
    """


@dataclass(eq=False, repr=False)
class CustomSockopt(betterproto2.Message):
    system: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    network: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    level: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    opt: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)

    value: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)

    type: "str" = betterproto2.field(6, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet", "CustomSockopt", CustomSockopt
)


@dataclass(eq=False, repr=False)
class ProxyConfig(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    transport_layer_proxy: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.transport.internet", "ProxyConfig", ProxyConfig
)


@dataclass(eq=False, repr=False)
class SocketConfig(betterproto2.Message):
    """
    SocketConfig is options to be applied on network sockets.
    """

    mark: "int" = betterproto2.field(1, betterproto2.TYPE_INT32)
    """
    Mark of the connection. If non-zero, the value will be set to SO_MARK.
    """

    tfo: "int" = betterproto2.field(2, betterproto2.TYPE_INT32)
    """
    TFO is the state of TFO settings.
    """

    tproxy: "SocketConfigTProxyMode" = betterproto2.field(
        3, betterproto2.TYPE_ENUM, default_factory=lambda: SocketConfigTProxyMode(0)
    )
    """
    TProxy is for enabling TProxy socket option.
    """

    receive_original_dest_address: "bool" = betterproto2.field(
        4, betterproto2.TYPE_BOOL
    )
    """
    ReceiveOriginalDestAddress is for enabling IP_RECVORIGDSTADDR socket
    option. This option is for UDP only.
    """

    bind_address: "bytes" = betterproto2.field(5, betterproto2.TYPE_BYTES)

    bind_port: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)

    accept_proxy_protocol: "bool" = betterproto2.field(7, betterproto2.TYPE_BOOL)

    domain_strategy: "DomainStrategy" = betterproto2.field(
        8, betterproto2.TYPE_ENUM, default_factory=lambda: DomainStrategy(0)
    )

    dialer_proxy: "str" = betterproto2.field(9, betterproto2.TYPE_STRING)

    tcp_keep_alive_interval: "int" = betterproto2.field(10, betterproto2.TYPE_INT32)

    tcp_keep_alive_idle: "int" = betterproto2.field(11, betterproto2.TYPE_INT32)

    tcp_congestion: "str" = betterproto2.field(12, betterproto2.TYPE_STRING)

    interface: "str" = betterproto2.field(13, betterproto2.TYPE_STRING)

    v6only: "bool" = betterproto2.field(14, betterproto2.TYPE_BOOL)

    tcp_window_clamp: "int" = betterproto2.field(15, betterproto2.TYPE_INT32)

    tcp_user_timeout: "int" = betterproto2.field(16, betterproto2.TYPE_INT32)

    tcp_max_seg: "int" = betterproto2.field(17, betterproto2.TYPE_INT32)

    penetrate: "bool" = betterproto2.field(18, betterproto2.TYPE_BOOL)

    tcp_mptcp: "bool" = betterproto2.field(19, betterproto2.TYPE_BOOL)

    custom_sockopt: "list[CustomSockopt]" = betterproto2.field(
        20, betterproto2.TYPE_MESSAGE, repeated=True
    )

    address_port_strategy: "AddressPortStrategy" = betterproto2.field(
        21, betterproto2.TYPE_ENUM, default_factory=lambda: AddressPortStrategy(0)
    )


default_message_pool.register_message(
    "xray.transport.internet", "SocketConfig", SocketConfig
)


@dataclass(eq=False, repr=False)
class StreamConfig(betterproto2.Message):
    address: "__common__net__.IpOrDomain | None" = betterproto2.field(
        8, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(9, betterproto2.TYPE_UINT32)

    protocol_name: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)
    """
    Effective network.
    """

    transport_settings: "list[TransportConfig]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )

    security_type: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Type of security. Must be a message name of the settings proto.
    """

    security_settings: "list[__common__serial__.TypedMessage]" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Transport security settings. They can be either TLS or REALITY.
    """

    socket_settings: "SocketConfig | None" = betterproto2.field(
        6, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.transport.internet", "StreamConfig", StreamConfig
)


@dataclass(eq=False, repr=False)
class TransportConfig(betterproto2.Message):
    protocol_name: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)
    """
    Transport protocol name.
    """

    settings: "__common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Specific transport protocol settings.
    """


default_message_pool.register_message(
    "xray.transport.internet", "TransportConfig", TransportConfig
)


from ...common import net as __common__net__
from ...common import serial as __common__serial__



from enum import StrEnum

from grpclib.exceptions import GRPCError

from xray_cfo.xray.app.proxyman.command import AddOutboundRequest, AddOutboundResponse

from .._base import BaseService


class OutProtocol(StrEnum):
    blackhole = "blackhole"
    freedom = "freedom"


class HandlerAddOutboundService(BaseService):

    async def _add_outbound(
        self, message: AddOutboundRequest
    ) -> AddOutboundResponse | None:
        try:
            return await self.handler_stub.add_outbound(message)
        except GRPCError as e:
            return None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: common/serial/typed_message.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("TypedMessage",)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class TypedMessage(betterproto2.Message):
    """
    TypedMessage is a serialized proto message along with its type name.
    """

    type: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    The name of the message type, retrieved from protobuf API.
    """

    value: "bytes" = betterproto2.field(2, betterproto2.TYPE_BYTES)
    """
    Serialized proto message.
    """


default_message_pool.register_message(
    "xray.common.serial", "TypedMessage", TypedMessage
)



from enum import StrEnum

from pydantic import Field

from .._base import XBase


class DomainStrategy(StrEnum):
    AsIs = "AsIs"
    UseIP = "UseIP"
    UseIPv6v4 = "UseIPv6v4"
    UseIPv6 = "UseIPv6"
    UseIPv4v6 = "UseIPv4v6"
    UseIPv4 = "UseIPv4"
    ForceIP = "ForceIP"
    ForceIPv6v4 = "ForceIPv6v4"
    ForceIPv6 = "ForceIPv6"
    ForceIPv4v6 = "ForceIPv4v6"
    ForceIPv4 = "ForceIPv4"


class Noise(XBase):
    type: str
    packet: str
    delay: str


class XFreedomOut(XBase):
    domain_strategy: DomainStrategy = Field(
        DomainStrategy.AsIs, serialization_alias="domainStrategy"
    )
    redirect: str = ""
    user_level: int = Field(0, serialization_alias="userLevel")
    fragment: dict | None = None
    noises: list[Noise] = Field(default_factory=list)

    proxy_protocol: int = Field(0, serialization_alias="proxyProtocol")



from enum import StrEnum

from pydantic import Field

from .._base import XBase


def get_default_alpn():
    return ["h2", "http/1.1"]


class TLSFingerprint(StrEnum):
    chrome = "chrome"
    firefox = "firefox"
    safari = "safari"
    ios = "ios"
    android = "android"
    edge = "edge"
    f360 = "360"
    qq = "qq"
    random = "random"
    randomized = "randomized"
    unsafe = "unsafe"


class TLSCurvePreferences(StrEnum):
    CurveP256 = "CurveP256"
    CurveP384 = "CurveP384"
    CurveP521 = "CurveP521"
    X25519 = "X25519"
    x25519Kyber768Draft00 = "x25519Kyber768Draft00"


def get_default_curve_preferences():
    return ["X25519"]


class TLSCertificateUsage(StrEnum):
    encipherment = "encipherment"
    verify = "verify"
    issue = "issue"


class TLSCertificate(XBase):
    ocsp_stapling: int = Field(3600, serialization_alias="ocspStapling")
    one_time_loading: bool = Field(False, serialization_alias="oneTimeLoading")
    usage: TLSCertificateUsage = TLSCertificateUsage.encipherment
    build_chain: bool = Field(False, serialization_alias="buildChain")
    certificate_file: str | None = Field(None, serialization_alias="certificateFile")
    key_file: str | None = Field(None, serialization_alias="keyFile")
    certificate: list[str] | None = None
    key: list[str] | None = None


class XTLSSettings(XBase):
    server_name: str = Field(serialization_alias="serverName")
    server_name_to_verify: str | None = Field(
        None, serialization_alias="serverNameToVerify"
    )
    reject_unknown_sni: bool = Field(False, serialization_alias="rejectUnknownSni")
    allow_insecure: bool = Field(True, serialization_alias="allowInsecure")
    alpn: list[str] = Field(default_factory=get_default_alpn)
    min_version: str = Field("1.2", serialization_alias="minVersion")
    max_version: str = Field("1.3", serialization_alias="maxVersion")
    cipher_suites: str | None = Field(None, serialization_alias="CipherSuites")
    certificates: list[TLSCertificate] = Field(default_factory=list)
    disable_system_root: bool = Field(False, serialization_alias="disableSystemRoot")
    enable_session_resumption: bool = Field(
        False, serialization_alias="enableSessionResumption"
    )
    fingerprint: TLSFingerprint = TLSFingerprint.chrome
    pinned_peer_certificate_chain_sha256: list[str] | None = Field(
        serialization_alias="pinnedPeerCertificateChainSha256"
    )
    curve_preferences: list[TLSCurvePreferences] = Field(
        default_factory=get_default_curve_preferences,
        serialization_alias="curvePreferences",
    )
    master_key_log: str | None = Field(None, serialization_alias="masterKeyLog")



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/shadowsocks_2022/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Account",
    "ClientConfig",
    "MultiUserServerConfig",
    "RelayDestination",
    "RelayServerConfig",
    "ServerConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    key: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.proxy.shadowsocks_2022", "Account", Account)


@dataclass(eq=False, repr=False)
class ClientConfig(betterproto2.Message):
    address: "__common__net__.IpOrDomain | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)

    method: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    key: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)

    udp_over_tcp: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)

    udp_over_tcp_version: "int" = betterproto2.field(6, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.proxy.shadowsocks_2022", "ClientConfig", ClientConfig
)


@dataclass(eq=False, repr=False)
class MultiUserServerConfig(betterproto2.Message):
    method: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    key: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    users: "list[__common__protocol__.User]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )

    network: "list[__common__net__.Network]" = betterproto2.field(
        4, betterproto2.TYPE_ENUM, repeated=True
    )


default_message_pool.register_message(
    "xray.proxy.shadowsocks_2022", "MultiUserServerConfig", MultiUserServerConfig
)


@dataclass(eq=False, repr=False)
class RelayDestination(betterproto2.Message):
    key: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    address: "__common__net__.IpOrDomain | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    port: "int" = betterproto2.field(3, betterproto2.TYPE_UINT32)

    email: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)

    level: "int" = betterproto2.field(5, betterproto2.TYPE_INT32)


default_message_pool.register_message(
    "xray.proxy.shadowsocks_2022", "RelayDestination", RelayDestination
)


@dataclass(eq=False, repr=False)
class RelayServerConfig(betterproto2.Message):
    method: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    key: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    destinations: "list[RelayDestination]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )

    network: "list[__common__net__.Network]" = betterproto2.field(
        4, betterproto2.TYPE_ENUM, repeated=True
    )


default_message_pool.register_message(
    "xray.proxy.shadowsocks_2022", "RelayServerConfig", RelayServerConfig
)


@dataclass(eq=False, repr=False)
class ServerConfig(betterproto2.Message):
    method: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    key: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    email: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    level: "int" = betterproto2.field(4, betterproto2.TYPE_INT32)

    network: "list[__common__net__.Network]" = betterproto2.field(
        5, betterproto2.TYPE_ENUM, repeated=True
    )


default_message_pool.register_message(
    "xray.proxy.shadowsocks_2022", "ServerConfig", ServerConfig
)


from ...common import net as __common__net__
from ...common import protocol as __common__protocol__



from enum import StrEnum

from pydantic import Field

from xray_cfo.models.xray.transport import XStreamSettings

from .._base import XBase
from .blackhole import BlackHoleOut
from .freedom import XFreedomOut


class XUDPProxyUDP443(StrEnum):
    reject = "reject"
    allow = "allow"
    skip = "skip"


class MuxObject(XBase):
    enabled: bool = True
    concurrency: int = 8
    xudp_concurrency: int = Field(16, serialization_alias="xudpConcurrency")
    xudp_proxy_udp_443: XUDPProxyUDP443 = Field(
        XUDPProxyUDP443.reject, serialization_alias="xudpProxyUDP443"
    )


class XOutboundObject(XBase):
    send_through: str = Field("0.0.0.0", serialization_alias="sendThrough")
    protocol: str
    tag: str
    settings: BlackHoleOut | XFreedomOut
    stream_settings: XStreamSettings | None = Field(
        None, serialization_alias="streamSettings"
    )
    proxy_settings: dict | None = Field(None, serialization_alias="proxySettings")
    mux: MuxObject | None = None



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vmess/inbound/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "DefaultConfig",
    "DetourConfig",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    user: "list[___common__protocol__.User]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )

    default: "DefaultConfig | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    detour: "DetourConfig | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    4 is for legacy setting
    """


default_message_pool.register_message("xray.proxy.vmess.inbound", "Config", Config)


@dataclass(eq=False, repr=False)
class DefaultConfig(betterproto2.Message):
    level: "int" = betterproto2.field(2, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.proxy.vmess.inbound", "DefaultConfig", DefaultConfig
)


@dataclass(eq=False, repr=False)
class DetourConfig(betterproto2.Message):
    to: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.proxy.vmess.inbound", "DetourConfig", DetourConfig
)


from ....common import protocol as ___common__protocol__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: transport/internet/kcp/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "ConnectionReuse",
    "DownlinkCapacity",
    "EncryptionSeed",
    "Mtu",
    "ReadBuffer",
    "Tti",
    "UplinkCapacity",
    "WriteBuffer",
)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    mtu: "Mtu | None" = betterproto2.field(1, betterproto2.TYPE_MESSAGE, optional=True)

    tti: "Tti | None" = betterproto2.field(2, betterproto2.TYPE_MESSAGE, optional=True)

    uplink_capacity: "UplinkCapacity | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    downlink_capacity: "DownlinkCapacity | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )

    congestion: "bool" = betterproto2.field(5, betterproto2.TYPE_BOOL)

    write_buffer: "WriteBuffer | None" = betterproto2.field(
        6, betterproto2.TYPE_MESSAGE, optional=True
    )

    read_buffer: "ReadBuffer | None" = betterproto2.field(
        7, betterproto2.TYPE_MESSAGE, optional=True
    )

    header_config: "___common__serial__.TypedMessage | None" = betterproto2.field(
        8, betterproto2.TYPE_MESSAGE, optional=True
    )

    seed: "EncryptionSeed | None" = betterproto2.field(
        10, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message("xray.transport.internet.kcp", "Config", Config)


@dataclass(eq=False, repr=False)
class ConnectionReuse(betterproto2.Message):
    enable: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.transport.internet.kcp", "ConnectionReuse", ConnectionReuse
)


@dataclass(eq=False, repr=False)
class DownlinkCapacity(betterproto2.Message):
    """
    Downlink capacity, in MB.
    """

    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.transport.internet.kcp", "DownlinkCapacity", DownlinkCapacity
)


@dataclass(eq=False, repr=False)
class EncryptionSeed(betterproto2.Message):
    """
    Pre-shared secret between client and server. It is used for traffic obfuscation.
    Note that if seed is absent in the config, the traffic will still be obfuscated,
    but by a predefined algorithm.
    """

    seed: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.transport.internet.kcp", "EncryptionSeed", EncryptionSeed
)


@dataclass(eq=False, repr=False)
class Mtu(betterproto2.Message):
    """
    Maximum Transmission Unit, in bytes.
    """

    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.transport.internet.kcp", "MTU", Mtu)


@dataclass(eq=False, repr=False)
class ReadBuffer(betterproto2.Message):
    size: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)
    """
    Buffer size in bytes.
    """


default_message_pool.register_message(
    "xray.transport.internet.kcp", "ReadBuffer", ReadBuffer
)


@dataclass(eq=False, repr=False)
class Tti(betterproto2.Message):
    """
    Transmission Time Interview, in milli-sec.
    """

    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.transport.internet.kcp", "TTI", Tti)


@dataclass(eq=False, repr=False)
class UplinkCapacity(betterproto2.Message):
    """
    Uplink capacity, in MB.
    """

    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message(
    "xray.transport.internet.kcp", "UplinkCapacity", UplinkCapacity
)


@dataclass(eq=False, repr=False)
class WriteBuffer(betterproto2.Message):
    size: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)
    """
    Buffer size in bytes.
    """


default_message_pool.register_message(
    "xray.transport.internet.kcp", "WriteBuffer", WriteBuffer
)


from ....common import serial as ___common__serial__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/trojan/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Account",
    "ClientConfig",
    "Fallback",
    "ServerConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Account(betterproto2.Message):
    password: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message("xray.proxy.trojan", "Account", Account)


@dataclass(eq=False, repr=False)
class ClientConfig(betterproto2.Message):
    server: "list[__common__protocol__.ServerEndpoint]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.proxy.trojan", "ClientConfig", ClientConfig)


@dataclass(eq=False, repr=False)
class Fallback(betterproto2.Message):
    name: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    alpn: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    path: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    type: "str" = betterproto2.field(4, betterproto2.TYPE_STRING)

    dest: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)

    xver: "int" = betterproto2.field(6, betterproto2.TYPE_UINT64)


default_message_pool.register_message("xray.proxy.trojan", "Fallback", Fallback)


@dataclass(eq=False, repr=False)
class ServerConfig(betterproto2.Message):
    users: "list[__common__protocol__.User]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )

    fallbacks: "list[Fallback]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.proxy.trojan", "ServerConfig", ServerConfig)


from ...common import protocol as __common__protocol__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/wireguard/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "DeviceConfigDomainStrategy",
    "DeviceConfig",
    "PeerConfig",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


class DeviceConfigDomainStrategy(betterproto2.Enum):
    FORCE_IP = 0

    FORCE_IP4 = 1

    FORCE_IP6 = 2

    FORCE_IP46 = 3

    FORCE_IP64 = 4


@dataclass(eq=False, repr=False)
class DeviceConfig(betterproto2.Message):
    secret_key: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    endpoint: "list[str]" = betterproto2.field(
        2, betterproto2.TYPE_STRING, repeated=True
    )

    peers: "list[PeerConfig]" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, repeated=True
    )

    mtu: "int" = betterproto2.field(4, betterproto2.TYPE_INT32)

    num_workers: "int" = betterproto2.field(5, betterproto2.TYPE_INT32)

    reserved: "bytes" = betterproto2.field(6, betterproto2.TYPE_BYTES)

    domain_strategy: "DeviceConfigDomainStrategy" = betterproto2.field(
        7, betterproto2.TYPE_ENUM, default_factory=lambda: DeviceConfigDomainStrategy(0)
    )

    is_client: "bool" = betterproto2.field(8, betterproto2.TYPE_BOOL)

    no_kernel_tun: "bool" = betterproto2.field(9, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.proxy.wireguard", "DeviceConfig", DeviceConfig
)


@dataclass(eq=False, repr=False)
class PeerConfig(betterproto2.Message):
    public_key: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    pre_shared_key: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)

    endpoint: "str" = betterproto2.field(3, betterproto2.TYPE_STRING)

    keep_alive: "int" = betterproto2.field(4, betterproto2.TYPE_UINT32)

    allowed_ips: "list[str]" = betterproto2.field(
        5, betterproto2.TYPE_STRING, repeated=True
    )


default_message_pool.register_message("xray.proxy.wireguard", "PeerConfig", PeerConfig)



from typing import TYPE_CHECKING

from xray_cfo.xray.app.stats.command import StatsServiceStub
from xray_cfo.xray.core.app.observatory.command import ObservatoryServiceStub
from xray_cfo.xray.transport.internet.grpc.encoding import GrpcServiceStub

if TYPE_CHECKING:
    from grpclib.client import Channel

from xray_cfo.xray.app.log.command import LoggerServiceStub
from xray_cfo.xray.app.proxyman.command import HandlerServiceStub
from xray_cfo.xray.app.router.command import RoutingServiceStub


class BaseService:
    _channel: "Channel"

    def __init__(self):
        self.grpc_stub = GrpcServiceStub(self._channel)
        self.handler_stub = HandlerServiceStub(self._channel)
        self.logger_stub = LoggerServiceStub(self._channel)
        self.observatory_stub = ObservatoryServiceStub(self._channel)
        self.routing_stub = RoutingServiceStub(self._channel)
        self.stats_stub = StatsServiceStub(self._channel)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proxy/vless/outbound/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = ("Config",)

from dataclasses import dataclass

import betterproto2

from .....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    vnext: "list[___common__protocol__.ServerEndpoint]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message("xray.proxy.vless.outbound", "Config", Config)


from ....common import protocol as ___common__protocol__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/policy/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "Policy",
    "PolicyBuffer",
    "PolicyStats",
    "PolicyTimeout",
    "Second",
    "SystemPolicy",
    "SystemPolicyStats",
)

from dataclasses import dataclass

import betterproto2

from ....message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    level: "dict[int, Policy]" = betterproto2.field(
        1,
        betterproto2.TYPE_MAP,
        map_meta=betterproto2.map_meta(
            betterproto2.TYPE_UINT32, betterproto2.TYPE_MESSAGE
        ),
    )

    system: "SystemPolicy | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message("xray.app.policy", "Config", Config)


@dataclass(eq=False, repr=False)
class Policy(betterproto2.Message):
    timeout: "PolicyTimeout | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    stats: "PolicyStats | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    buffer: "PolicyBuffer | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message("xray.app.policy", "Policy", Policy)


@dataclass(eq=False, repr=False)
class PolicyBuffer(betterproto2.Message):
    connection: "int" = betterproto2.field(1, betterproto2.TYPE_INT32)
    """
    Buffer size per connection, in bytes. -1 for unlimited buffer.
    """


default_message_pool.register_message("xray.app.policy", "Policy.Buffer", PolicyBuffer)


@dataclass(eq=False, repr=False)
class PolicyStats(betterproto2.Message):
    user_uplink: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)

    user_downlink: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)

    user_online: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)


default_message_pool.register_message("xray.app.policy", "Policy.Stats", PolicyStats)


@dataclass(eq=False, repr=False)
class PolicyTimeout(betterproto2.Message):
    """
    Timeout is a message for timeout settings in various stages, in seconds.
    """

    handshake: "Second | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )

    connection_idle: "Second | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )

    uplink_only: "Second | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )

    downlink_only: "Second | None" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.policy", "Policy.Timeout", PolicyTimeout
)


@dataclass(eq=False, repr=False)
class Second(betterproto2.Message):
    value: "int" = betterproto2.field(1, betterproto2.TYPE_UINT32)


default_message_pool.register_message("xray.app.policy", "Second", Second)


@dataclass(eq=False, repr=False)
class SystemPolicy(betterproto2.Message):
    stats: "SystemPolicyStats | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message("xray.app.policy", "SystemPolicy", SystemPolicy)


@dataclass(eq=False, repr=False)
class SystemPolicyStats(betterproto2.Message):
    inbound_uplink: "bool" = betterproto2.field(1, betterproto2.TYPE_BOOL)

    inbound_downlink: "bool" = betterproto2.field(2, betterproto2.TYPE_BOOL)

    outbound_uplink: "bool" = betterproto2.field(3, betterproto2.TYPE_BOOL)

    outbound_downlink: "bool" = betterproto2.field(4, betterproto2.TYPE_BOOL)


default_message_pool.register_message(
    "xray.app.policy", "SystemPolicy.Stats", SystemPolicyStats
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: core/config.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "Config",
    "InboundHandlerConfig",
    "OutboundHandlerConfig",
)

from dataclasses import dataclass

import betterproto2

from ...message_pool import default_message_pool

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    """
    Config is the master config of Xray. Xray takes this config as input and
    functions accordingly.
    """

    inbound: "list[InboundHandlerConfig]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Inbound handler configurations. Must have at least one item.
    """

    outbound: "list[OutboundHandlerConfig]" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Outbound handler configurations. Must have at least one item. The first
    item is used as default for routing.
    """

    app: "list[_common__serial__.TypedMessage]" = betterproto2.field(
        4, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    App is for configurations of all features in Xray. A feature must
    implement the Feature interface, and its config type must be registered
    through common.RegisterConfig.
    """

    extension: "list[_common__serial__.TypedMessage]" = betterproto2.field(
        6, betterproto2.TYPE_MESSAGE, repeated=True
    )
    """
    Configuration for extensions. The config may not work if corresponding
    extension is not loaded into Xray. Xray will ignore such config during
    initialization.
    """


default_message_pool.register_message("xray.core", "Config", Config)


@dataclass(eq=False, repr=False)
class InboundHandlerConfig(betterproto2.Message):
    """
    InboundHandlerConfig is the configuration for inbound handler.
    """

    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    Tag of the inbound handler. The tag must be unique among all inbound
    handlers
    """

    receiver_settings: "_common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for how this inbound proxy is handled.
    """

    proxy_settings: "_common__serial__.TypedMessage | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for inbound proxy. Must be one of the inbound proxies.
    """


default_message_pool.register_message(
    "xray.core", "InboundHandlerConfig", InboundHandlerConfig
)


@dataclass(eq=False, repr=False)
class OutboundHandlerConfig(betterproto2.Message):
    """
    OutboundHandlerConfig is the configuration for outbound handler.
    """

    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)
    """
    Tag of this outbound handler.
    """

    sender_settings: "_common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for how to dial connection for this outbound handler.
    """

    proxy_settings: "_common__serial__.TypedMessage | None" = betterproto2.field(
        3, betterproto2.TYPE_MESSAGE, optional=True
    )
    """
    Settings for this outbound proxy. Must be one of the outbound proxies.
    """

    expire: "int" = betterproto2.field(4, betterproto2.TYPE_INT64)
    """
    If not zero, this outbound will be expired in seconds. Not used for now.
    """

    comment: "str" = betterproto2.field(5, betterproto2.TYPE_STRING)
    """
    Comment of this outbound handler. Not used for now.
    """


default_message_pool.register_message(
    "xray.core", "OutboundHandlerConfig", OutboundHandlerConfig
)


from ..common import serial as _common__serial__



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: app/proxyman/command/command.proto
# plugin: python-betterproto2
# This file has been @generated

__all__ = (
    "AddInboundRequest",
    "AddInboundResponse",
    "AddOutboundRequest",
    "AddOutboundResponse",
    "AddUserOperation",
    "AlterInboundRequest",
    "AlterInboundResponse",
    "AlterOutboundRequest",
    "AlterOutboundResponse",
    "Config",
    "GetInboundUserRequest",
    "GetInboundUserResponse",
    "GetInboundUsersCountResponse",
    "ListInboundsRequest",
    "ListInboundsResponse",
    "ListOutboundsRequest",
    "ListOutboundsResponse",
    "RemoveInboundRequest",
    "RemoveInboundResponse",
    "RemoveOutboundRequest",
    "RemoveOutboundResponse",
    "RemoveUserOperation",
    "HandlerServiceStub",
    "HandlerServiceBase",
)

from dataclasses import dataclass
from typing import TYPE_CHECKING

import betterproto2
import grpclib
from betterproto2.grpc.grpclib_server import ServiceBase

from .....message_pool import default_message_pool

if TYPE_CHECKING:
    import grpclib.server
    from betterproto2.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline

betterproto2.check_compiler_version("0.5.1")


@dataclass(eq=False, repr=False)
class AddInboundRequest(betterproto2.Message):
    inbound: "___core__.InboundHandlerConfig | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "AddInboundRequest", AddInboundRequest
)


@dataclass(eq=False, repr=False)
class AddInboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "AddInboundResponse", AddInboundResponse
)


@dataclass(eq=False, repr=False)
class AddOutboundRequest(betterproto2.Message):
    outbound: "___core__.OutboundHandlerConfig | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "AddOutboundRequest", AddOutboundRequest
)


@dataclass(eq=False, repr=False)
class AddOutboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "AddOutboundResponse", AddOutboundResponse
)


@dataclass(eq=False, repr=False)
class AddUserOperation(betterproto2.Message):
    user: "___common__protocol__.User | None" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "AddUserOperation", AddUserOperation
)


@dataclass(eq=False, repr=False)
class AlterInboundRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    operation: "___common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "AlterInboundRequest", AlterInboundRequest
)


@dataclass(eq=False, repr=False)
class AlterInboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "AlterInboundResponse", AlterInboundResponse
)


@dataclass(eq=False, repr=False)
class AlterOutboundRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    operation: "___common__serial__.TypedMessage | None" = betterproto2.field(
        2, betterproto2.TYPE_MESSAGE, optional=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "AlterOutboundRequest", AlterOutboundRequest
)


@dataclass(eq=False, repr=False)
class AlterOutboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "AlterOutboundResponse", AlterOutboundResponse
)


@dataclass(eq=False, repr=False)
class Config(betterproto2.Message):
    pass


default_message_pool.register_message("xray.app.proxyman.command", "Config", Config)


@dataclass(eq=False, repr=False)
class GetInboundUserRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)

    email: "str" = betterproto2.field(2, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.proxyman.command", "GetInboundUserRequest", GetInboundUserRequest
)


@dataclass(eq=False, repr=False)
class GetInboundUserResponse(betterproto2.Message):
    users: "list[___common__protocol__.User]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "GetInboundUserResponse", GetInboundUserResponse
)


@dataclass(eq=False, repr=False)
class GetInboundUsersCountResponse(betterproto2.Message):
    count: "int" = betterproto2.field(1, betterproto2.TYPE_INT64)


default_message_pool.register_message(
    "xray.app.proxyman.command",
    "GetInboundUsersCountResponse",
    GetInboundUsersCountResponse,
)


@dataclass(eq=False, repr=False)
class ListInboundsRequest(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "ListInboundsRequest", ListInboundsRequest
)


@dataclass(eq=False, repr=False)
class ListInboundsResponse(betterproto2.Message):
    inbounds: "list[___core__.InboundHandlerConfig]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "ListInboundsResponse", ListInboundsResponse
)


@dataclass(eq=False, repr=False)
class ListOutboundsRequest(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "ListOutboundsRequest", ListOutboundsRequest
)


@dataclass(eq=False, repr=False)
class ListOutboundsResponse(betterproto2.Message):
    outbounds: "list[___core__.OutboundHandlerConfig]" = betterproto2.field(
        1, betterproto2.TYPE_MESSAGE, repeated=True
    )


default_message_pool.register_message(
    "xray.app.proxyman.command", "ListOutboundsResponse", ListOutboundsResponse
)


@dataclass(eq=False, repr=False)
class RemoveInboundRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.proxyman.command", "RemoveInboundRequest", RemoveInboundRequest
)


@dataclass(eq=False, repr=False)
class RemoveInboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "RemoveInboundResponse", RemoveInboundResponse
)


@dataclass(eq=False, repr=False)
class RemoveOutboundRequest(betterproto2.Message):
    tag: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.proxyman.command", "RemoveOutboundRequest", RemoveOutboundRequest
)


@dataclass(eq=False, repr=False)
class RemoveOutboundResponse(betterproto2.Message):
    pass


default_message_pool.register_message(
    "xray.app.proxyman.command", "RemoveOutboundResponse", RemoveOutboundResponse
)


@dataclass(eq=False, repr=False)
class RemoveUserOperation(betterproto2.Message):
    email: "str" = betterproto2.field(1, betterproto2.TYPE_STRING)


default_message_pool.register_message(
    "xray.app.proxyman.command", "RemoveUserOperation", RemoveUserOperation
)


class HandlerServiceStub(betterproto2.ServiceStub):
    async def add_inbound(
        self,
        message: "AddInboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AddInboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/AddInbound",
            message,
            AddInboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def remove_inbound(
        self,
        message: "RemoveInboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "RemoveInboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/RemoveInbound",
            message,
            RemoveInboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def alter_inbound(
        self,
        message: "AlterInboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AlterInboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/AlterInbound",
            message,
            AlterInboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_inbounds(
        self,
        message: "ListInboundsRequest | None" = None,
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "ListInboundsResponse":
        if message is None:
            message = ListInboundsRequest()

        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/ListInbounds",
            message,
            ListInboundsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_inbound_users(
        self,
        message: "GetInboundUserRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetInboundUserResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/GetInboundUsers",
            message,
            GetInboundUserResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_inbound_users_count(
        self,
        message: "GetInboundUserRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "GetInboundUsersCountResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/GetInboundUsersCount",
            message,
            GetInboundUsersCountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def add_outbound(
        self,
        message: "AddOutboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AddOutboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/AddOutbound",
            message,
            AddOutboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def remove_outbound(
        self,
        message: "RemoveOutboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "RemoveOutboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/RemoveOutbound",
            message,
            RemoveOutboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def alter_outbound(
        self,
        message: "AlterOutboundRequest",
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "AlterOutboundResponse":
        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/AlterOutbound",
            message,
            AlterOutboundResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def list_outbounds(
        self,
        message: "ListOutboundsRequest | None" = None,
        *,
        timeout: "float | None" = None,
        deadline: "Deadline | None" = None,
        metadata: "MetadataLike | None" = None,
    ) -> "ListOutboundsResponse":
        if message is None:
            message = ListOutboundsRequest()

        return await self._unary_unary(
            "/xray.app.proxyman.command.HandlerService/ListOutbounds",
            message,
            ListOutboundsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


from .... import core as ___core__
from ....common import protocol as ___common__protocol__
from ....common import serial as ___common__serial__


class HandlerServiceBase(ServiceBase):
    async def add_inbound(self, message: "AddInboundRequest") -> "AddInboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def remove_inbound(
        self, message: "RemoveInboundRequest"
    ) -> "RemoveInboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def alter_inbound(
        self, message: "AlterInboundRequest"
    ) -> "AlterInboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_inbounds(
        self, message: "ListInboundsRequest"
    ) -> "ListInboundsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_inbound_users(
        self, message: "GetInboundUserRequest"
    ) -> "GetInboundUserResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_inbound_users_count(
        self, message: "GetInboundUserRequest"
    ) -> "GetInboundUsersCountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def add_outbound(
        self, message: "AddOutboundRequest"
    ) -> "AddOutboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def remove_outbound(
        self, message: "RemoveOutboundRequest"
    ) -> "RemoveOutboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def alter_outbound(
        self, message: "AlterOutboundRequest"
    ) -> "AlterOutboundResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def list_outbounds(
        self, message: "ListOutboundsRequest"
    ) -> "ListOutboundsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_add_inbound(
        self, stream: "grpclib.server.Stream[AddInboundRequest, AddInboundResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.add_inbound(request)
        await stream.send_message(response)

    async def __rpc_remove_inbound(
        self,
        stream: "grpclib.server.Stream[RemoveInboundRequest, RemoveInboundResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.remove_inbound(request)
        await stream.send_message(response)

    async def __rpc_alter_inbound(
        self, stream: "grpclib.server.Stream[AlterInboundRequest, AlterInboundResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.alter_inbound(request)
        await stream.send_message(response)

    async def __rpc_list_inbounds(
        self, stream: "grpclib.server.Stream[ListInboundsRequest, ListInboundsResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_inbounds(request)
        await stream.send_message(response)

    async def __rpc_get_inbound_users(
        self,
        stream: "grpclib.server.Stream[GetInboundUserRequest, GetInboundUserResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_inbound_users(request)
        await stream.send_message(response)

    async def __rpc_get_inbound_users_count(
        self,
        stream: "grpclib.server.Stream[GetInboundUserRequest, GetInboundUsersCountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_inbound_users_count(request)
        await stream.send_message(response)

    async def __rpc_add_outbound(
        self, stream: "grpclib.server.Stream[AddOutboundRequest, AddOutboundResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.add_outbound(request)
        await stream.send_message(response)

    async def __rpc_remove_outbound(
        self,
        stream: "grpclib.server.Stream[RemoveOutboundRequest, RemoveOutboundResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.remove_outbound(request)
        await stream.send_message(response)

    async def __rpc_alter_outbound(
        self,
        stream: "grpclib.server.Stream[AlterOutboundRequest, AlterOutboundResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.alter_outbound(request)
        await stream.send_message(response)

    async def __rpc_list_outbounds(
        self,
        stream: "grpclib.server.Stream[ListOutboundsRequest, ListOutboundsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.list_outbounds(request)
        await stream.send_message(response)

    def __mapping__(self) -> "dict[str, grpclib.const.Handler]":
        return {
            "/xray.app.proxyman.command.HandlerService/AddInbound": grpclib.const.Handler(
                self.__rpc_add_inbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                AddInboundRequest,
                AddInboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/RemoveInbound": grpclib.const.Handler(
                self.__rpc_remove_inbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                RemoveInboundRequest,
                RemoveInboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/AlterInbound": grpclib.const.Handler(
                self.__rpc_alter_inbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                AlterInboundRequest,
                AlterInboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/ListInbounds": grpclib.const.Handler(
                self.__rpc_list_inbounds,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListInboundsRequest,
                ListInboundsResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/GetInboundUsers": grpclib.const.Handler(
                self.__rpc_get_inbound_users,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetInboundUserRequest,
                GetInboundUserResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/GetInboundUsersCount": grpclib.const.Handler(
                self.__rpc_get_inbound_users_count,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetInboundUserRequest,
                GetInboundUsersCountResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/AddOutbound": grpclib.const.Handler(
                self.__rpc_add_outbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                AddOutboundRequest,
                AddOutboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/RemoveOutbound": grpclib.const.Handler(
                self.__rpc_remove_outbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                RemoveOutboundRequest,
                RemoveOutboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/AlterOutbound": grpclib.const.Handler(
                self.__rpc_alter_outbound,
                grpclib.const.Cardinality.UNARY_UNARY,
                AlterOutboundRequest,
                AlterOutboundResponse,
            ),
            "/xray.app.proxyman.command.HandlerService/ListOutbounds": grpclib.const.Handler(
                self.__rpc_list_outbounds,
                grpclib.const.Cardinality.UNARY_UNARY,
                ListOutboundsRequest,
                ListOutboundsResponse,
            ),
        }



from enum import StrEnum

from pydantic import Field

from .._base import XBase


class RealityFingerprint(StrEnum):
    chrome = "chrome"
    firefox = "firefox"
    safari = "safari"
    ios = "ios"
    android = "android"
    edge = "edge"
    f360 = "360"
    qq = "qq"
    random = "random"
    randomized = "randomized"
    unsafe = "unsafe"


class XRealitySettingsIN(XBase):
    show: bool = False
    target: str
    xver: int = 0
    server_names: list[str] = Field(serialization_alias="serverNames")
    private_key: str = Field(serialization_alias="privateKey")
    min_client_ver: str | None = Field(serialization_alias="minClientVer")
    max_client_ver: str | None = Field(serialization_alias="maxClientVer")
    max_time_diff: int = Field(0, serialization_alias="maxTimeDiff")
    short_ids: list[str] | None = Field(None, serialization_alias="shortIds")


class XRealitySettingsOUT(XBase):
    fingerprint: RealityFingerprint = RealityFingerprint.chrome
    server_name: str = Field(serialization_alias="serverName")
    public_key: str = Field(serialization_alias="publicKey")
    short_id: str | None = Field(serialization_alias="shortId")
    spider_x: str = Field(serialization_alias="spiderX")
